EXPORTS
	NP_GetEntryPoints   @1
	NP_Initialize       @2
	NP_Shutdown         @3
	DllCanUnloadNow	     PRIVATE
	DllGetClassObject    PRIVATE
	DllRegisterServer    PRIVATE
	DllUnregisterServer  PRIVATE
	DumpIDL		     PRIVATE
